import { showMessage, hideMessage } from 'react-native-flash-message';
import { View, TouchableOpacity, ActivityIndicator, Image } from 'react-native';
import Text from './text';
import TextButton from './text-button';
import { ErrorSymbolIcon } from './icons/error-symbol';
import { ClearSymbolIcon } from './icons/clear-symbol';
import Spinner from './spinner';

type SnackBarVariant = 'default' | 'danger' | 'success' | 'thumbnail';

interface SnackBarProps {
  message?: string;
  variant?: 'default' | 'danger' | 'success' | 'thumbnail';
  loading?: boolean;
  buttonText?: string;
  duration?: number;
  styleTheme?: 'dark' | 'white';
  rightTextButton?: boolean;
  thumbnailProps?: {
    imageUrl?: string;
    title: string;
    description: string;
  };
  className?: string;
}

// 스타일 및 아이콘 설정
const getVariantStyles = (variant: SnackBarVariant) => {
  switch (variant) {
    case 'success':
      return {
        backgroundColor: 'bg-emerald-50',
        textColor: 'emerald-700',
        icon: 'check-circle',
      };
    case 'danger':
      return {
        backgroundColor: 'bg-red-50',
        textColor: 'red-700',
        icon: 'error',
      };
    default:
      return {
        backgroundColor: 'bg-gray-50',
        textColor: 'gray-700',
        icon: 'info',
      };
  }
};

// App.tsx에서 사용할 스타일 설정
export const flashMessageStyle = {
  backgroundColor: 'transparent', // 배경 투명
  padding: 0,
  margin: 0,
};

const CustomSnackBarContent = ({
  message,
  variant = 'default',
  loading = false,
  buttonText = '확인',
  styleTheme = 'dark',
  rightTextButton = false,
  thumbnailProps,
  className,
}: Omit<SnackBarProps, 'duration'>) => {
  const styles = getVariantStyles(variant);

  const renderIcon = () => {
    if (loading) {
      return (
        <View>
          <Spinner size="xsmall" />
        </View>
      );
    }

    switch (variant) {
      case 'danger':
        return <ErrorSymbolIcon />;
      case 'success':
        return <ClearSymbolIcon />;
      default:
        return null;
    }
  };

  if (variant === 'thumbnail' && thumbnailProps) {
    return (
      <View
        className={`
          flex-row items-center justify-between 
          min-h-[60px]
          p-[16px]
          ${styleTheme === 'dark' ? 'bg-gray-900' : 'bg-white'}
          rounded-xl 
          shadow-lg
          ${className}
        `}
      >
        <View
          className={`flex-row items-center flex-1 ${rightTextButton ? 'mr-[8px]' : ''}`}
        >
          <Image
            source={{
              uri: thumbnailProps.imageUrl || '',
            }}
            className="w-[42px] h-[42px] rounded-[6px]"
          />
          <View className="ml-[12px] flex-1">
            <Text
              face="body"
              size="large"
              weight="bold"
              textColor={styleTheme === 'dark' ? 'white' : 'black'}
            >
              {thumbnailProps.title}
            </Text>
            <Text
              face="body"
              size="small"
              weight="regular"
              textColor={styleTheme === 'dark' ? 'gray-400' : 'gray-600'}
              className="mt-[2px]"
            >
              {thumbnailProps.description}
            </Text>
          </View>
        </View>

        {rightTextButton && (
          <TextButton
            text={buttonText}
            variant="brand"
            underline={true}
            onPress={() => hideMessage()}
            size="medium"
          />
        )}
      </View>
    );
  }

  return (
    <View
      className={`
      flex-row items-center justify-between 
      min-h-[60px]
      p-[16px]
      ${styleTheme === 'dark' ? 'bg-gray-900' : 'bg-white'}
      rounded-xl 
      shadow-lg
      ${className}
    `}
    >
      <View className="flex-row items-center flex-1">
        {renderIcon()}
        <Text
          face="body"
          size="large"
          weight="regular"
          textColor={styleTheme === 'dark' ? 'white' : 'black'}
          className="ml-[6px] flex-1"
        >
          {message}
        </Text>
      </View>

      {rightTextButton && (
        <TextButton
          text={buttonText}
          variant="brand"
          underline={true}
          onPress={() => hideMessage()}
          size="medium"
        />
      )}
    </View>
  );
};

let isSnackBarVisible = false;

const showSnackBar = ({
  message,
  variant = 'default',
  loading = false,
  buttonText = '확인',
  duration = 3000,
  rightTextButton = false,
  styleTheme = 'dark',
  thumbnailProps,
  className,
}: SnackBarProps) => {
  const commonConfig = {
    message: '',
    duration: duration,
    backgroundColor: 'transparent',
    hideStatusBar: false,
    swipeEnabled: true,
    swipeDirection: ['up', 'left', 'right'], // 모든 방향 스와이프 허용
    renderCustomContent: () => (
      <CustomSnackBarContent
        message={message}
        variant={variant}
        styleTheme={styleTheme}
        loading={loading}
        buttonText={buttonText}
        rightTextButton={rightTextButton}
        thumbnailProps={thumbnailProps}
        className={className}
      />
    ),
  };

  if (!isSnackBarVisible) {
    isSnackBarVisible = true;
    showMessage({
      ...commonConfig,
      onHide: () => {
        isSnackBarVisible = false;
      },
    });
  } else {
    showMessage(commonConfig);
  }
};

export default showSnackBar;
